function [g0,g,h] = collect(p,x);  
% function [g0,g,h] = collect(p,x);  
%
% DESCRIPTION 
%   Collect p(x,y) into the form g0(x)+g1(x)*h1(y) + ... + gN(x)*hN(y)
%   where each hi(y) is a unique monomial in y and each gi(x) are
%   polynomials in x.
%
% INPUTS 
%   p: 1-by-1 polynomial in variables x and y.
%   x: variables of p to collect into polynomials with coefficients given
%      by monomials in y. x can either be a polynomial vector or
%      a cell array of strings of variable names. 
%
% OUTPUTS 
%   g0: 1-by-1 polynomial in x.
%   g: 1-by-N vector of polynomials in x.
%   h: 1-by-N vector of monomials in y.
%   
%   g0, g, and h satisfy p(x,y) = g0(x)+g1(x)*h1(y) + ... + gN(x)*hN(y)
%   where gi(x)/hi(y) are given by the i^th columns of g/h. 
%
% SYNTAX 
%   [g0,g,h] = collect(p,x);
%
% EXAMPLE
%   pvar x1 x2 y1 y2;
%   p = 13+x1^2*y1-5*x1^2*y2^3+6*x1*x2*y1+8*x1;
%   x = [x1;x2];
%   [g0,g,h] = collect(p,x)
%   p-(g0+g*h')

% 1/29/08  PJS     Initial Coding
% 5/21/09  PJS     Updates for speed

% XXX Error checking

% Get list of "x" variables as a cell string
p = polynomial(p);
if isa(x,'polynomial');
    x = x.varname;
end

% Rewrite degree matrix of p in common set of variables
[allvars,tmp,idx] = unique( [p.varname; x] ); 
nv = length(allvars);

[ntp,nvp] = size(p.degmat);
pdeg = sparse(ntp,nv);
pdeg(:,idx(1:nvp)) = p.degmat;

% Split out degree matrices associated with x and y variables
nx = length(x);
xidx = idx(end-nx+1:end);
xdegmat =  pdeg(:,xidx);
yidx = 1:nv;
yidx(xidx) = [];

%[temp,xidx] = intersect(allvars,x);
%xdegmat =  pdeg(:,xidx);
%[temp,yidx] = setdiff(allvars,x);
y = allvars(yidx);

if isempty(yidx)
    g0 = p;
    g = polynomial(0);
    h = polynomial(0);
    return;
else
    ydegmat =  pdeg(:,yidx);
end

% Find unique list of monomials in y
[hdegmat,I,J]=unique(ydegmat,'rows'); 
lh = size(hdegmat,1);

% Collect polynomials in x whose coefficients are monomials in y
% g = polynomial( zeros(1,lh) );
% h = polynomial( zeros(1,lh) );
% 
% L.type = '()';
% gc = []; gd = [];
% for i1=1:lh
%     L.subs = {i1};
%     
%     gidx = find( J==i1 );    
%     gi = polynomial( p.coefficient(gidx), xdegmat(gidx,:), x, [1 1]);    
%     g = subsasgn(g,L,gi);
% 
%      hi = polynomial( 1 , hdegmat(i1,:), y, [1 1]);    
%      h = subsasgn(h,L,hi);
% end

% chkval = 0; % skip validity check
% g = polynomial(sparse(1:length(J),J,p.coefficient),xdegmat,x,[1 lh],chkval);
% h = polynomial( speye(lh), hdegmat, y, [1 lh],chkval);


% h, as constructed may have a constant monomial, hi(y)==1.
% Pull out the polynomial, g0(x), associated with this monomial.
g0idx = find( sum(hdegmat,2)== 0 );
chkval = 0; % skip validity check in polynomial constructor
if isempty(g0idx)
    g0 = polynomial(0);
    g = polynomial(sparse(1:length(J),J,p.coefficient),xdegmat,x,[1 lh],chkval);
    h = polynomial( speye(lh), hdegmat, y, [1 lh],chkval);

    % Combine Terms 
    g = combine(g);
else
    idx = [1:(g0idx-1) (g0idx+1):lh];

    gcoef = sparse(1:length(J),J,p.coefficient);
    g0 = polynomial(gcoef(:,g0idx),xdegmat,x,[1 1],chkval);
    g = polynomial(gcoef(:,idx),xdegmat,x,[1 lh-1],chkval);

    hdegmat(g0idx,:) = [];
    h = polynomial( speye(lh-1), hdegmat, y, [1 lh-1],chkval);

    % Combine Terms 
    g0 = combine(g0);
    g = combine(g);
    
%     L.type = '()';
%     L.subs = {g0idx};
%     g0 = subsref(g,L); 
%     
%     idx = setdiff(1:lh,g0idx);
%     idx = [1:(g0idx-1) (g0idx+1):lh];
%     L.subs = {idx};
%     g = subsref(g,L); 
%     h = subsref(h,L);     
end

% % Combine Terms 
% g0 = combine(g0);
% g = combine(g);
% h = combine(h);
