function p = polynomial(coefficient,degmat,varname,matdim,chkval);
% function P = polynomial(Coefficient,Degmat,Varname,Matdim);
%
% DESCRIPTION 
%   Creates a polynomial or a matrix of polynomials.
%   
% INPUTS 
%   Coefficient: coefficients of each monomial.
%   Degmat: degrees of each monomial
%   Varname: names of variables
%   Matdim: dimensions of the polynomial matrix  
%
% OUTPUT 
%   P: polynomial object
%
% SYNTAX 
%   P=polynomial  
%     Creates an empty polynomial object.
%   P=polynomial(Coefficient)
%     If Coefficient is a real matrix of dimension NxM, then P is
%     an NxM constant polynomial. 
%   P=polynomial(Varname)
%     If Var is an NxM cell array of strings, then P is an NxM polynomial
%     whose entries are the variables specified in Var.
%   P=polynomial(Coefficient)
%     If Coefficient is a polynomial object, then P=Coefficient.
%   P=polynomial(Coefficient,Degmat,Varname,Matdim)  
%     If P is an NxM polynomial that is the sum of T terms in V
%     variables, the inputs should be specified as: 
%       Coefficient is a Tx(N*M) sparse matrix.
%          The coefficients of the (i,j) entry of the polynomial
%          matrix are a Tx1 vector stored in the i+*N*(j-1) column of 
%          Coefficient.
%       Degmat is a TxV sparse matrix of natural numbers.  Row t 
%          gives the degrees of each variable for the t^th term.
%       Varname is a Vx1 cell array with entry v giving the name of
%          variable v. For a constant polynomial, varname is an
%          empty 1x1 cell.  
%       Matdim is a 1x2 vector of the matrix dimensions, [N M].
  
% 6/7/2002:   PJS  Initial Coding
% 6/8/2002:   PJS  Allow Matrices of Polynomials
% 10/30/2002: PJS  Input is structure not cell arrays
% 11/5/2002:  PJS  Coefficients stored as sparse 3D array.
% 4/20/2009:  PJS  Changed Coefficient matrix from (T*N*M)x1 to Tx(N*M)
% 5/20/2009:  PJS  Added chkval (=0 to skip validity check, else = 1)
  
superiorto('double');

%--------------------------------------%
%    Fill in empty arguments
%--------------------------------------%
if nargin == 0
    p.coefficient = sparse([]);
    p.degmat = sparse([]);
    p.varname = {};
    p.matdim = [0 0];
    p=class(p,'polynomial');
    return
elseif nargin==1 
    if isa(coefficient,'polynomial')  
        % If input is already a polynomial, pass it back  
        p = coefficient;
        return;
    elseif isa(coefficient,'double') && isreal(coefficient) && ...
        ndims(coefficient)==2
    
        % If input is a real double, then convert to a polynomial object
        nt = 1;
        szc = size(coefficient);
        p.coefficient = reshape(coefficient,[1 szc(1)*szc(2)]);
        p.degmat = sparse(1,0);
        p.varname  = {};    
        p.matdim = szc;

        p=class(p,'polynomial');
        return  
    elseif iscellstr(coefficient) && ndims(coefficient)==2
        V = coefficient;
        szV = size(V);
        numV = szV(1)*szV(2);
        
        p.coefficient = speye(numV);
        p.degmat = speye(numV);
        p.varname = V(:);
        p.matdim = szV;
        p=class(p,'polynomial');
        return;
    else
        error(['For single input, argument must be a polynomial, '...
            'real double, or cell array of strings']);
    end
elseif nargin<4
    error(['Not enough inputs: P = polynomial(Coefficient,Degmat,'...
    'Varname,Matdim)']);
elseif nargin==4
    chkval=1;
end

%--------------------------------------%
%       Error Checking
%--------------------------------------%
p.coefficient = coefficient;
p.degmat = degmat;
p.varname = varname(:);
p.matdim = matdim(:)';

if chkval
    [flag,errormsg] = PVisvalid(p);
else
    % Skip validity check -- This is used by internal functions, e.g.
    %  combine, for which validity check is not needed.  This speeds
    %  up the call to polynomial
    flag = 1;
end

if flag==1
    % p is a valid polynomial object
    p.coefficient = sparse(p.coefficient);
    p.degmat = sparse(p.degmat);
    p=class(p,'polynomial');
    %p=combine(p);
else
    % p is not a valid polynomial object.
    error(errormsg);
end








