function b = subs(a,old,new)
% function B = subs(A,Old,New);
%
% DESCRIPTION 
%  Symbolic Substitution.
%   
% INPUTS 
%   A: polynomial
%   Old: an array of polynomial variables or a cell array of 
%          polynomial variables
%   New: polynomial or double or a cell array of polynomials and/or 
%          doubles.
%   (Either A and/or New must be 1x1)
%
% OUTPUTS  
%   B:  polynomial.  Use the 'double' command to depromote a constant
%        polynomial to a double.
%    
% SYNTAX 
%   B = subs(A);
%       -Replaces all variables in A with values in the workspace of
%        the calling function.
%   B = subs(A,Old,New);
%       -Replaces Old with New in the symbolic expression A.
%       -If Old is a vector of pvars and New is double of the same 
%        size, each element of Old is replaced with the corresponding 
%        element of New.
%       -If Old and New are cell arrays of the same size, each element 
%       of Old is replaced by the corresponding element of New. 
%       -If A and Old are scalars and New is an array or cell array, 
%       the scalars are expanded to produce an array result.
%   B = subs(A,New);
%       -If New is an N-by-2 cell array then this is equivalent
%        to B=subs(A,OldNew(:,1),OldNew(:,2));
%       -Otherwise this equivalent to B=subs(A,A.varname{1},New);
%
% EXAMPLE
%  pvar x1 x2
%  x=[x1;x2];
%  p=2*x1^4+2*x1^3*x2-x1^2*x2^2+5*x2^4
%  subs(p,x,[1;2])
%  subs(p,x,{[0 1 1]; [1 0 2]})
%  subs(p,{x1, [0 1 1]; x2 [1 0 2]})
%

% 3/25/2003: PJS  Initial Coding  
  
% Promote a to polynomial 
a = polynomial(a);
sza = a.matdim;

% Handle each of the three syntaxes
if nargin == 1   
    % Syntax: B = subs(A);
    
    new = cell(0);
    nva = length(a.varname);
    cnt = 1;
    for i1 = 1:nva

        if evalin('caller', ['isa(' a.varname{i1} ',''double'')'] );
            old{cnt} = a.varname{i1};
            new{cnt} = evalin('caller',a.varname{i1});
            cnt = cnt +1;
        end

        if evalin('caller', ['isa(' a.varname{i1} ',''polynomial'')'] );
            temp = evalin('caller',a.varname{i1});
            if temp~=polynomial(1,1,a.varname(i1),[1 1])
                old{cnt} = a.varname{i1};
                new{cnt} = temp;
                cnt = cnt+1;
            end
        end

    end

    if isempty(new) 
        b = a;
        return;
    else
        b=subs(a,old,new);
        return;
    end
  
elseif nargin == 2     
    if iscell(old) && size(old,2)==2
        % Syntax:  B = subs(A,OldNew);
        b=subs(a,old(:,1),old(:,2));
        return;
    else
        % Syntax:  B = subs(A,New);
        b=subs(a,a.varname{1},old);
        return;
    end
end

% Get dimensions
szold = size(old);
sznew = size(new);

%---------------------------------------------------------------------
% Handle the basic syntax: B = subs(A,Old,New);
%---------------------------------------------------------------------

% If new is a 1x1 cell array or a cell array of scalars then convert 
% to a double or polynomial array. If it is a cell array of arrays 
% then reformat and call subs using the array replacement syntax.

% If new is a cell array then convert to a double or polynomial array
if isa(new,'cell')
    isd = cellfun('isclass',new,'double');
    isp = cellfun('isclass',new,'polynomial');
    isp = isp | isd;
    
    if all(sza==[1 1]) && any(szold==1) && any(sznew==1)
        % e) a is 1-by-1, old is No-by-1, new is No-by-1 array of
        %         n-by-m --> b is n-by-m        
        % Reformat to array syntax:
        % c) a is Nr-by-1, old is n-by-1, new is n-by-m --> b is Nr-by-m
        tmpsz = cellfun(@size,new,'UniformOutput',false);
        tmpnew = [];
        L.type = '()';
        for i1=1:sznew(1)*sznew(2)
            if any(tmpsz{i1} ~= tmpsz{1})
                error('All cell array entries of new must have the same dimensions');
            else                
                L.subs = {':'};
                tmp = subsref(new{i1},L);
                %b = subsasgn(b,L,tmp);                
                tmpnew = [tmpnew tmp];
            end
        end
        b = subs(a,old(:),tmpnew');
        b = reshape(b,tmpsz{1,1});
        return;
    else
        if all( isd(:) )
            % Convert cell array of doubles to a matrix of doubles
            new = cell2mat(new);
        elseif all( isp(:) )
            % Convert cell array of polys/doubles to a poly array
            tmp = polynomial( zeros(sznew) );
            L.type = '()';            
            for i1=1:sznew(1)*sznew(2)
                L.subs = {i1};
                tmp = subsasgn(tmp,L,new{i1});                
                %tmp(i1) = new{i1};
            end
            new = tmp;
        else
            error(['New should be a polynomial, double or a ' ... 
                    'cell array of polynomials and/or doubles.']);
        end               
    end    
end

% Convert old to a cell array of chars
if ispvar(old)
    old = char(old);
elseif isa(old,'cell')
    if cellfun(@ispvar,old)
        old = cellfun(@char,old);
    elseif ~cellfun(@ischar,old)
        error(['Old should be an array of polynomial variables'  ...
	       'or a cell array of polynomial variables']);
    end
else
    error(['Old should be an array of polynomial variables'  ...
	       'or a cell array of polynomial variables']);
end

% Convert to:
%  a is Na-by-1
%  old is Nold-by-1
%  new is Nold-by-Npts
% Solve and then convert back to the right dimensions
% Cases:
%   a) a,old are 1-by-1 and new is n-by-m --> b is n-by-m
%   b) a is Nr-by-Nc, old is n-by-m, new is n-by-m --> b is Nr-by-Nc
%   c) a is Nr-by-1, old is n-by-1, new is n-by-m --> b is Nr-by-m
% XXX) a is 1-by-Nc, old is n-by-1, new is n-by-m --> b is m-by-Nc
%   e) a is 1-by-1, old is No-by-1, new is No-by-1 array of
%                 n-by-m --> b is n-by-m
% c and d are not allowed by symbolic subs. 
% Convert case a to case b by expansion
% Skip case d? If so, then we only have to deal with cases b and c
%   No--> a cannot be converted to b by expansion (watch dimensions)

% Perform numeric or symbolic substitution
[ismem,idxa]=ismember(a.varname,old(:));
if isa(new,'double') && all(ismem) && length(a.varname)==length(old(:)) 
    % Case 1: New is a double and all variables of A will be replaced.
    %   Result will be numeric and we can use peval for fast computation.    
    if all(sza==[1 1]) && all(szold == [1 1])
        % a) A/Old are 1-by-1 and New is n-by-m --> B is n-by-m
        b = peval(new(:)', a.coefficient,a.degmat);
        b = reshape(b,sznew);
    elseif all(sznew == szold)
        % b) A is r-by-c and  Old/New are n-by-m --> B is r-by-c
        new = new(idxa);
        acoef = a.coefficient;
        b = peval(new(:), acoef,a.degmat);
        b = reshape(b,sza);
    elseif sza(2)==1 && szold(1)==sznew(1) && szold(2)==1 
        % c) A is r-by-1, Old is n-by-1, New is n-by-m --> B is r-by-m
        new = new(idxa,:);
        acoef = a.coefficient;
        b = peval(new, acoef,a.degmat);
        b = reshape(b,sza(1),sznew(2));        
    else
        error('Dimensions of New, Old, and A are not compatible');
    end
    b = polynomial(b);
    return;
    
elseif isa(new,'double') 
    % Case 2: New is a double but with only some variables of A being
    %   replaced (peval cannot be used).
    if all(sza==[1 1]) && all(szold == [1 1])
        % a) A/Old are 1-by-1 and New is n-by-m --> B is n-by-m
        b = LOCAL_numericsubs(a,old,new);        
    elseif all(sznew == szold)
        % b) A is r-by-c and  Old/New are n-by-m --> B is r-by-c
        b = LOCAL_numericsubs(a,old(:),new(:));
    elseif sza(2)==1 && szold(1)==sznew(1) && szold(2)==1 
        % c) A is r-by-1, Old is n-by-1, New is n-by-m --> B is r-by-m
        b = polynomial( zeros(sza(1),sznew(2)) );
        L.type = '()';
        for i1=1:sznew(2)
            L.subs = {':',i1};
            newij = subsref(new,L);
            tmp = LOCAL_numericsubs(a,old,newij);            
            b = subsasgn(b,L,tmp);
        end
    else
        error('Dimensions of New, Old, and A are not compatible');
    end
    
    % Combine down
    b = combine(b);   
    return;    

    
elseif isa(new,'polynomial') % | isa(new,'double')  
    % Case 3: New is a polynomial. Use symbolic replacement
    if all(sza==[1 1]) && all(szold == [1 1])
        % a) A/Old are 1-by-1 and New is n-by-m --> B is n-by-m
        b = polynomial(zeros(sznew));
        L.type = '()';
        for i1 = 1:sznew(1)
            for i2= 1:sznew(2)
                L.subs = {i1,i2};
                newij = subsref(new,L);
                tmp = LOCAL_symbolicsubs(a,old,newij);
                b = subsasgn(b,L,tmp);
            end
        end
    elseif all(sznew == szold)
        % b) A is r-by-c and  Old/New are n-by-m --> B is r-by-c
        b = LOCAL_symbolicsubs(a,old(:),new(:));
    elseif sza(2)==1 && szold(1)==sznew(1) && szold(2)==1 
        % c) A is r-by-1, Old is n-by-1, New is n-by-m --> B is r-by-m
        b = polynomial( zeros(sza(1),sznew(2)) );
        L.type = '()';
        for i1=1:sznew(2)
            L.subs = {':',i1};
            newij = subsref(new,L);
            tmp = LOCAL_symbolicsubs(a,old,newij);            
            b = subsasgn(b,L,tmp);
        end
    else
        error('Dimensions of New, Old, and A are not compatible');
    end
    
    % Combine down
    b = combine(b);   
    return;    
else
    error(['New should be a polynomial, double or a ' ... 
            'cell array of polynomials and/or doubles.']);
end


%---------------------------------------------------------------------
% LOCAL_numericsubs
%---------------------------------------------------------------------
function b = LOCAL_numericsubs(a,old,new)

b=a;
szb = size(b);
szold = size(old);
ntb = size(b.degmat,1);
tempsz = szb(1)*szb(2);
bcoef = b.coefficient;

if all(szb==[1 1]) && all(szold == [1 1])
    szn = size(new);    
    b.matdim = szn;
    tempsz = szn(1)*szn(2);
    bcoef = repmat(b.coefficient,1,tempsz);    
    
    idx = strmatch(old{1},b.varname,'exact'); 
    if ~isempty(idx)
        tempdeg = b.degmat(:,idx);
        b.degmat(:,idx) = [];
        b.varname(idx)=[];  
        tempdeg = repmat(tempdeg,1,tempsz);
        tempnew = repmat(new(:)',ntb,1);        
        bcoef = bcoef.*(tempnew.^tempdeg); 
        b.coefficient=bcoef;
    end
else
%     for i1 = 1:length(old)
%         idx = strmatch(old{i1},b.varname,'exact'); 
%         if ~isempty(idx)
%             tempdeg = b.degmat(:,idx);
%             b.degmat(:,idx) = [];
%             b.varname(idx)=[];  
% 
%             tempdeg = repmat(tempdeg,1,tempsz);
%             tempnew = repmat(new(i1),ntb,tempsz);
%             bcoef = bcoef.*(tempnew.^tempdeg); 
%             b.coefficient=bcoef;
%         end
%     end
    
    [tf,idx]=ismember(old,b.varname);    
    idx = idx(tf);
    new = new(tf);

    tempdeg = b.degmat(:,idx);
    b.degmat(:,idx) = [];
    b.varname(idx)=[];  

    tempnew = repmat(new(:)',ntb,1);
    tempcoef = prod(tempnew.^tempdeg,2);    
    bcoef = bcoef.*repmat(tempcoef,[1 szb(1)*szb(2)]); 
    b.coefficient=bcoef;
end

%---------------------------------------------------------------------
% LOCAL_symbolicsubs
%---------------------------------------------------------------------
function b = LOCAL_symbolicsubs(a,old,new)

b=a;
ntb = size(b.degmat,1);
nvb = size(b.degmat,2);
bcoef = b.coefficient;
bdeg = b.degmat; 
bvar = b.varname;
bdim = b.matdim;   

for i1 = 1:length(old);
    temp = strmatch(old{i1},b.varname,'exact'); 
    if isempty(temp)
        idx(i1) = -1;
        %error(['Undefined variable ' old{i1}]);
    else
        idx(i1) = temp;
    end
end
idx2=setdiff(1:nvb,idx);

b=polynomial(0);
L.type = '()';
for i1 = 1:ntb        
    coef = bcoef(i1,:);
    if isempty(idx2)
        temppoly = polynomial(reshape(coef,bdim));
    else      
        %coef = coef(:);
        temppoly = polynomial(coef,bdeg(i1,idx2),bvar(idx2),bdim);
    end

    for i2 = 1:length(old)      
        if idx(i2)>0
            L.subs = {i2};
            newi = subsref(new,L);
            temppoly = temppoly*power(newi,bdeg(i1,idx(i2)));
        end
    end
    b=b+temppoly;
end

